@@ -35,6 +35,7 @@
   exception if a NaN distance value occurs.
 */
 
+#include <cmath> // for std::pow, std::sqrt
 #include <cstddef> // for std::ptrdiff_t
 #include <limits> // for std::numeric_limits<...>::infinity()
 #include <algorithm> // for std::fill_n
@@ -163,14 +164,6 @@ enum method_codes_vector {
   MAX_METHOD_VECTOR_CODE       = 3
 };
 
-enum {
-   // Return values
-  RET_SUCCESS        = 0,
-  RET_MEMORY_ERROR   = 1,
-  RET_STL_ERROR      = 2,
-  RET_UNKNOWN_ERROR  = 3
- };
-
 // self-destructing array pointer
 template <typename type>
 class auto_array_ptr{
@@ -213,17 +206,11 @@ public:
 struct node {
   t_index node1, node2;
   t_float dist;
+};
 
-  /*
-  inline bool operator< (const node a) const {
-    return this->dist < a.dist;
-  }
-  */
-
-  inline friend bool operator< (const node a, const node b) {
+inline bool operator< (const node a, const node b) {
     return (a.dist < b.dist);
   }
-};
 
 class cluster_result {
 private:
@@ -250,7 +237,7 @@ public:
 
   void sqrt() const {
     for (node * ZZ=Z; ZZ!=Z+pos; ++ZZ) {
-      ZZ->dist = ::sqrt(ZZ->dist);
+      ZZ->dist = std::sqrt(ZZ->dist);
     }
   }
 
@@ -260,14 +247,14 @@ public:
 
   void sqrtdouble(const t_float) const { // ignore the argument
     for (node * ZZ=Z; ZZ!=Z+pos; ++ZZ) {
-      ZZ->dist = ::sqrt(2*ZZ->dist);
+      ZZ->dist = std::sqrt(2*ZZ->dist);
     }
   }
 
   #ifdef R_pow
   #define my_pow R_pow
   #else
-  #define my_pow pow
+  #define my_pow std::pow
   #endif
 
   void power(const t_float p) const {
