@@ -23,6 +23,7 @@
 #define fc_isnan(X) ((X)!=(X))
 // There is ISNAN but it is so much slower on my x86_64 system with GCC!
 
+#include <cmath> // for std::abs
 #include <cstddef> // for std::ptrdiff_t
 #include <limits> // for std::numeric_limits<...>::infinity()
 #include <algorithm> // for std::stable_sort
@@ -426,7 +427,7 @@ private:
     double * p2 = x+i2*nc;
     for(j = 0 ; j < nc ; ++j) {
       if(both_non_NA(*p1, *p2)) {
-        dev = fabs(*p1 - *p2);
+        dev = std::abs(*p1 - *p2);
         if(!ISNAN(dev)) {
           if(dev > dist)
             dist = dev;
@@ -450,7 +451,7 @@ private:
     double * p2 = x+i2*nc;
     for(j = 0 ; j < nc ; ++j) {
       if(both_non_NA(*p1, *p2)) {
-        dev = fabs(*p1 - *p2);
+        dev = std::abs(*p1 - *p2);
         if(!ISNAN(dev)) {
           dist += dev;
           ++count;
@@ -474,8 +475,8 @@ private:
     double * p2 = x+i2*nc;
     for(j = 0 ; j < nc ; ++j) {
       if(both_non_NA(*p1, *p2)) {
-        sum = fabs(*p1 + *p2);
-        diff = fabs(*p1 - *p2);
+        sum = std::abs(*p1 + *p2);
+        diff = std::abs(*p1 - *p2);
         if (sum > DBL_MIN || diff > DBL_MIN) {
           dev = diff/sum;
 #if HAVE_DIAGNOSTIC
@@ -553,7 +554,7 @@ private:
       if(both_non_NA(*p1, *p2)) {
         dev = (*p1 - *p2);
         if(!ISNAN(dev)) {
-          dist += R_pow(fabs(dev), p);
+          dist += R_pow(std::abs(dev), p);
           ++count;
         }
       }
@@ -581,24 +582,20 @@ extern "C" {
         Input checks
       */
       // Parameter N: number of data points
-      PROTECT(N_);
       if (!IS_INTEGER(N_) || LENGTH(N_)!=1)
         Rf_error("'N' must be a single integer.");
-      const int N = *INTEGER_POINTER(N_);
+      const int N = INTEGER_VALUE(N_);
       if (N<2)
         Rf_error("N must be at least 2.");
-      const std::ptrdiff_t NN = static_cast<std::ptrdiff_t>(N)*(N-1)/2;
-      UNPROTECT(1); // N_
+      const R_xlen_t NN = static_cast<R_xlen_t>(N)*(N-1)/2;
 
       // Parameter method: dissimilarity index update method
-      PROTECT(method_);
       if (!IS_INTEGER(method_) || LENGTH(method_)!=1)
         Rf_error("'method' must be a single integer.");
-      const int method = *INTEGER_POINTER(method_) - 1; // index-0 based;
+      const int method = INTEGER_VALUE(method_) - 1; // index-0 based;
       if (method<MIN_METHOD_CODE || method>MAX_METHOD_CODE) {
         Rf_error("Invalid method index.");
       }
-      UNPROTECT(1); // method_
 
       // Parameter members: number of members in each node
       auto_array_ptr<t_float> members;
@@ -616,13 +613,13 @@ extern "C" {
             Rf_error("'members' must have length N.");
           const t_float * const m = NUMERIC_POINTER(members_);
           for (t_index i=0; i<N; ++i) members[i] = m[i];
-          UNPROTECT(1); // members
+          UNPROTECT(1); // members_
         }
       }
 
       // Parameter D_: dissimilarity matrix
       PROTECT(D_ = AS_NUMERIC(D_));
-      if (LENGTH(D_)!=NN)
+      if (XLENGTH(D_)!=NN)
         Rf_error("'D' must have length (N \\choose 2).");
       const double * const D = NUMERIC_POINTER(D_);
       // Make a working copy of the dissimilarity array
@@ -630,7 +627,7 @@ extern "C" {
       auto_array_ptr<double> D__;
       if (method!=METHOD_METR_SINGLE) {
         D__.init(NN);
-        for (std::ptrdiff_t i=0; i<NN; ++i)
+        for (R_xlen_t i=0; i<NN; ++i)
           D__[i] = D[i];
       }
       UNPROTECT(1); // D_
@@ -751,25 +748,21 @@ extern "C" {
       */
 
       // Parameter method: dissimilarity index update method
-      PROTECT(method_);
       if (!IS_INTEGER(method_) || LENGTH(method_)!=1)
         Rf_error("'method' must be a single integer.");
-      int method = *INTEGER_POINTER(method_) - 1; // index-0 based;
-      if (method<METHOD_VECTOR_SINGLE || method>METHOD_VECTOR_MEDIAN) {
+      int method = INTEGER_VALUE(method_) - 1; // index-0 based;
+      if (method<MIN_METHOD_VECTOR_CODE || method>MAX_METHOD_VECTOR_CODE) {
         Rf_error("Invalid method index.");
       }
-      UNPROTECT(1); // method_
 
       // Parameter metric
-      PROTECT(metric_);
       if (!IS_INTEGER(metric_) || LENGTH(metric_)!=1)
         Rf_error("'metric' must be a single integer.");
-      int metric = *INTEGER_POINTER(metric_) - 1; // index-0 based;
+      int metric = INTEGER_VALUE(metric_) - 1; // index-0 based;
       if (metric<0 || metric>5 ||
           (method!=METHOD_VECTOR_SINGLE && metric!=0) ) {
         Rf_error("Invalid metric index.");
       }
-      UNPROTECT(1); // metric_
 
       // data array
       PROTECT(X_ = AS_NUMERIC(X_));
@@ -785,9 +778,8 @@ extern "C" {
       // Make a working copy of the dissimilarity array
       // for all methods except "single".
       double * X__ = NUMERIC_POINTER(X_);
-      // Copy the input array and change it from Fortran-contiguous  style
-      // to C-contiguous style
-      // (Waste of memory for 'single'; the other methods need a copy
+      // Copy the input array and change it from Fortran-contiguous style
+      // to C-contiguous style.
       auto_array_ptr<double> X(LENGTH(X_));
       for (std::ptrdiff_t i=0; i<N; ++i)
         for (std::ptrdiff_t j=0; j<dim; ++j)
@@ -809,24 +801,22 @@ extern "C" {
             Rf_error("The length of 'members' must be the same as the number of data points.");
           const t_float * const m = NUMERIC_POINTER(members_);
           for (t_index i=0; i<N; ++i) members[i] = m[i];
-          UNPROTECT(1); // members
+          UNPROTECT(1); // members_
         }
       }
 
       // Parameter p
-      PROTECT(p_);
       double p = 0;
       if (metric==METRIC_R_MINKOWSKI) {
         if (!IS_NUMERIC(p_) || LENGTH(p_)!=1)
           Rf_error("'p' must be a single floating point number.");
-        p = *NUMERIC_POINTER(p_);
+        p = NUMERIC_VALUE(p_);
       }
       else {
         if (p_ != R_NilValue) {
           Rf_error("No metric except 'minkowski' allows a 'p' parameter.");
         }
       }
-      UNPROTECT(1); // p_
 
       /* The generic_linkage_vector_alternative algorithm uses labels
          N,N+1,... for the new nodes, so we need a table which node is
